package cn.ninegame.genericframework.hybrid;

import android.os.Build;
import android.os.Bundle;
import android.os.Looper;
import android.util.Log;
import android.webkit.WebView;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Set;

import cn.ninegame.genericframework.basic.Environment;
import cn.ninegame.genericframework.basic.FrameworkFacade;
import cn.ninegame.genericframework.basic.IResultListener;
import cn.ninegame.genericframework.ui.NativeWebView;

/**
 * <p>Title: ucweb</p>
 * <p/>
 * <p>Description: </p>
 * JS 和 Java 相互调用的中间层
 *
 * JS 调用 Java：  客户端通过 bundle.getSerializable(); 获取一个参数HashMap
 * <p>Copyright: Copyright (c) 2016</p>
 * <p/>
 * <p>Company: ucweb.com</p>
 *
 * @author ZhuZhiFan (zhifan.zzf@alibaba-inc.com)
 * @date 2016/10/23
 */
public class NativeApp {
    private static final String TAG = "NativeApp";

    public final static String CALL_JS_IF_METHOD_EXIST = "if(window.NativeApp && NativeApp.%s) NativeApp.%s(";
    public final static String JS_PREFIX = "javascript:";
    public final static String ON_CALLBACK_METHOD = "onCallback";
    public final static String ON_EVENT_METHOD = "onEvent";

    public static final String ARG_MESSAGE = "message";
    public static final String ARG_ARGS = "args";
    public static final String ARG_FROM_NATIVE_APP = "fromNativeApp";
    public static final String ARG_ERROR = "error";
    public static final String ARG_CALLBACK_ID = "callbackId";
    public static final String ARG_DATA = "data";
    public static final String ARG_CODE = "code";
    public static final String ARG_WEBVIEW = "webview";
    public static final String ARG_STACK = "stack";

    public static final String KEY_HANDLE_BY_DEFAULT = "handle_by_default";
    // 同步消息后缀
    private static final String SYN_MSG_SUFFIX = "_sync";

    //方法找不到
    public static final String METHOD_NOT_FOUND = "404";

    private static List<INativeAppInterceptor> sInterceptorList = new ArrayList<INativeAppInterceptor>();

    /**
     * JS 调用 Java的统一入口方法
     * @param webView webView容器
     * @param cmd
     *               JS 传递过来的指令数组， 根据msg参数分发给对应的处理者。
     * @return
     *              <p>返回JSON格式的数组，个数跟 cmd 指令的指令数组一致</p>
     *              <p>同步消息返回的数据存在{@link #ARG_DATA} 节点， 如果有异常，异常消息存在{@link #ARG_ERROR} 节点</p>
     *              <p>异步消息返回空JSON对象</p>
     */

    public static String prompt(WebView webView, String cmd) {
        if (cmd != null && !"".equals(cmd)) {
            JSONArray jsonArray = null;
            try {
                jsonArray = new JSONArray(cmd);
                ArrayList<Object> result = new ArrayList<Object>(jsonArray.length());

                for (int i = 0; i < jsonArray.length(); i++) {
                    JSONObject json = jsonArray.getJSONObject(i);
                    String msg = json.optString(ARG_MESSAGE);
                    if (msg.endsWith(SYN_MSG_SUFFIX)){
                        result.add(NativeApp.callNativeSync(webView, msg, json.optString(ARG_ARGS)));
                    } else {
                        NativeApp.callNative(webView, msg, json.optString(ARG_ARGS), json.optInt(ARG_CALLBACK_ID));
                        result.add(new Object());
                    }
                }

                String jsonStr = JSON.toJSONString(result);
                Log.d("JSInterface", "jsonStr = " + jsonStr);
                return jsonStr;

            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        return null;
    }

    /**
     * JS异步调用Java方法
     * @param webView
     * @param msg
     * @param args
     * @param callback
     */
    public static void callNative(final WebView webView, String msg, String args, final int callback) {
        Log.d(TAG, String.format("callNative msg=%s, args=%s, callback=%d", msg, args, callback));
        Bundle argsBundle = new Bundle();
        argsBundle.putBoolean(ARG_FROM_NATIVE_APP, true);

        HashMap<String, String> argsMap = null;
        try {
            argsMap = JSON.parseObject(args, new TypeReference<HashMap<String, String>>() {});
        } catch (Exception e) {
            e.printStackTrace();
        }

        INativeAppInterceptor interceptor = matchInterceptor(msg);

        if (interceptor == null) {
            if (argsMap != null) {
                argsBundle.putSerializable(ARG_ARGS, argsMap);
            }
            Environment env = FrameworkFacade.getInstance().getEnvironment();

            if (env != null) {
                if (NativeAppEventController.MSG_REGISTER_EVENT.equals(msg) || NativeAppEventController.MSG_UNREGISTER_EVENT.equals(msg)) {
                    // 注册和反注册事件
                    env.sendMessageForResult(msg, argsBundle, ((NativeWebView) webView).getEventResultListener());
                } else {
                    env.sendMessageForResult(msg, argsBundle, new IResultListener() {
                        @Override
                        public void onResult(Bundle bundle) {
                            boolean handleByDefault = bundle.getBoolean(KEY_HANDLE_BY_DEFAULT, false);
                            NativeApp.callJS(webView, NativeApp.buildOnCallbackScript(callback, JSON.toJSONString(genCallback(handleByDefault, bundle))));
                        }
                    });
                }
            }
        } else {
            interceptor.onIntercept(webView, msg, argsMap, new IResultListener() {
                @Override
                public void onResult(Bundle bundle) {
                    boolean handleByDefault = bundle.getBoolean(KEY_HANDLE_BY_DEFAULT, false);
                    NativeApp.callJS(webView, NativeApp.buildOnCallbackScript(callback, JSON.toJSONString(genCallback(handleByDefault, bundle))));
                }
            });
        }
    }

    /**
     * JS同步调用Java方法
     * <p>args 参数会被解析成HashMap，放进Bundle中，业务放可以通过 bundle.getSerializable({@link #ARG_ARGS}) 获取</p>
     * @param msg
     * @param args JSON格式参数
     * @return
     */
    public static Object callNativeSync(WebView webView, String msg, String args){
        Log.d(TAG, "callNativeSync");

        HashMap<String, String> argsMap = null;
        try {
            argsMap = JSON.parseObject(args, new TypeReference<HashMap<String, String>>() {});
        } catch (Exception e) {
            e.printStackTrace();
        }

        INativeAppInterceptor interceptor = matchInterceptor(msg);

        if (interceptor == null) {
            Environment env = FrameworkFacade.getInstance().getEnvironment();
            if (env != null) {
                Bundle argsBundle = new Bundle();
                argsBundle.putBoolean(ARG_FROM_NATIVE_APP, true);
                if (argsMap != null) {
                    argsBundle.putSerializable(ARG_ARGS, argsMap);
                }
                Bundle bundle = env.sendMessageSync(msg, argsBundle);
                return genCallback(bundle.getBoolean(KEY_HANDLE_BY_DEFAULT, false), bundle);
            } else {
                return null;
            }
        } else {
            Bundle bundle = interceptor.onInterceptSyn(webView, msg, argsMap);
            return genCallback(bundle.getBoolean(KEY_HANDLE_BY_DEFAULT, false), bundle);
        }


    }

    /**
     * 生成返回的数据
     * @param handleByDefault 是否是默认处理器处理
     */
    private static Object genCallback(boolean handleByDefault, Bundle bundle) {
        HashMap<String, HashMap<String, Object>> resultHasMap = new HashMap<>();

        if (handleByDefault) {
            HashMap<String, Object> errorMap = new HashMap<String, Object>();
            errorMap.put(ARG_CODE, METHOD_NOT_FOUND);
            errorMap.put(ARG_MESSAGE, "method not found");
            resultHasMap.put(ARG_ERROR, errorMap);
        } else {
            Serializable errorSerializable = bundle.getSerializable(ARG_ERROR);
            if (errorSerializable == null) {
                resultHasMap.put(ARG_DATA, bundle2HashMap(bundle));
            } else {
                resultHasMap.put(ARG_ERROR, (HashMap<String, Object>)errorSerializable);
            }
        }

        return resultHasMap;
    }

    /**
     * 调用 JS 的 onEvent 方法
     * @param webView
     * @param bundle
     */
    public static void callJSOnEvent(WebView webView, Bundle bundle){
        Log.d("NativeOnEvent", "调用 JS 的 onEvent 方法");
        NativeApp.callJS(webView, NativeApp.buildOnEventScript(bundle.getString(
                NativeAppEventController.KEY_EVENT_ID), genEventData(bundle)));
    }

    /**
     * 构建调用 JS onCallback 方法的脚本
     * @param callbackId
     * @param jsonStr
     * @return
     */
    private static String buildOnCallbackScript(int callbackId, String jsonStr){
        StringBuilder sb = new StringBuilder().append(String.format(CALL_JS_IF_METHOD_EXIST, ON_CALLBACK_METHOD,ON_CALLBACK_METHOD));
        sb.append(callbackId).append(",'").append(jsonStr).append("')");

        return sb.toString();
    }

    private static String genEventData(Bundle bundle){
//        HashMap<String, String> data = (HashMap<String, String>) bundle.getSerializable(ARG_DATA);
//        if (data == null) {
//            return JSON.toJSONString(new Object());
//        } else {
//            return JSON.toJSONString(data);
//        }

        return JSON.toJSONString(bundle2HashMap(bundle));
    }

    /**
     * 构建调用 JS onEvent方法的脚本
     * @param type
     * @param jsonStr
     * @return
     */
    private static String buildOnEventScript(String type, String jsonStr){
        StringBuilder sb = new StringBuilder().append(String.format(CALL_JS_IF_METHOD_EXIST, ON_EVENT_METHOD,ON_EVENT_METHOD));
        sb.append("'").append(type).append("',").append(jsonStr).append(")");

        return sb.toString();
    }

    /**
     * Java 调用 JS 方法
     * @param webview
     * @param script
     */
    public static void callJS(final WebView webview, final String script) {
        if (Looper.myLooper() == Looper.getMainLooper()) {
            callJSExecutor(webview, script);
        } else {
            webview.post(new Runnable() {
                @Override
                public void run() {
                    callJSExecutor(webview, script);
                }
            });
        }
    }

    /**
     * Java 调用 JS 方法 具体实现
     * @param webView
     * @param js
     */
    public static void callJSExecutor(WebView webView, String js) {
        try {
            if (Build.VERSION.SDK_INT < Build.VERSION_CODES.KITKAT) {
                if (!js.startsWith(JS_PREFIX)) {
                    js = JS_PREFIX + js;
                }
                synchronized (NativeApp.class) {
                    try {
                        webView.loadUrl(js);
                    } catch (Throwable e) {
                        e.printStackTrace();
                    }
                }
            } else {
                String replaced = js.replace(JS_PREFIX, "");
                synchronized (NativeApp.class) {
                    webView.evaluateJavascript(replaced, null);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 匹配拦截器
     */
    static INativeAppInterceptor matchInterceptor(String msg){
        if (!sInterceptorList.isEmpty()){
            for (INativeAppInterceptor interceptor : sInterceptorList) {
                if (interceptor.matchIntercept(msg)){
                    return interceptor;
                }
            }
        }
        return null;
    }

    /**
     * 增加消息拦截器
     * @param interceptor
     */
    public static void addInterceptor(INativeAppInterceptor interceptor){
        if (!sInterceptorList.contains(interceptor)) {
            sInterceptorList.add(interceptor);
        }
    }

    /**
     * 移除消息拦截器
     * @param interceptor
     */
    public static void removeInterceptor(INativeAppInterceptor interceptor){
        if (sInterceptorList.contains(interceptor)) {
            sInterceptorList.remove(interceptor);
        }
    }

    /**
     * 判断字符串是否属于JavaScript 脚本
     * @param script
     * @return
     */
    public static boolean isJSString(String script){
        return script.startsWith(CALL_JS_IF_METHOD_EXIST) || script.startsWith(JS_PREFIX);
    }

    static HashMap<String, Object> bundle2HashMap(Bundle bundle){
        HashMap<String, Object> data = new HashMap<String, Object>();
        if (bundle != null) {
            Set<String> keys = bundle.keySet();
            for (String key : keys) {
                data.put(key, bundle.get(key));
            }
        }
        return data;
    }
}
